# flake8: noqa

from authress.utils.jwt_manager import JwtManager
from authress.utils.package_version_provider import PackageVersionProvider
from authress.utils.service_client_token_provider import ServiceClientTokenProvider


# coding: utf-8

"""
    Authress

    <p> <h2>Introduction</h2> <p>Welcome to the Authress Authorization API. <br>The Authress REST API provides the operations and resources necessary to create records, assign permissions, and verify any user in your platform.</p> <p><ul>   <li>Manage multitenant platforms and create user tenants for SSO connections.</li>   <li>Create records to assign roles and resources to grant access for users.</li>   <li>Check user access control by calling the authorization API at the right time.</li>   <li>Configure service clients to securely access services in your platform.</li> </ul></p> <p>For more in-depth scenarios check out the <a href=\"https://authress.io/knowledge-base\" target=\"_blank\">Authress knowledge base</a>.</p> </p>  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: support@authress.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401
import io
import warnings

try:
    from pydantic.v1 import validate_arguments, ValidationError, Field, StrictStr, conint, constr, validator
except ImportError:
    from pydantic import validate_arguments, ValidationError, Field, StrictStr, conint, constr, validator
from typing_extensions import Annotated

from typing import Optional

from authress.models.role import Role
from authress.models.role_collection import RoleCollection

from authress.api_response import ApiResponse
from authress.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class RolesApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        self.api_client = api_client

    @validate_arguments
    def create_role(self, role : Role, **kwargs) -> Role:  # noqa: E501
        """Create role  # noqa: E501

        Creates a role with permissions.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_role(role, async_req=True)
        >>> result = thread.get()

        :param role: (required)
        :type role: Role
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Role
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the create_role_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.create_role_with_http_info(role, **kwargs)  # noqa: E501

    @validate_arguments
    def create_role_with_http_info(self, role : Role, **kwargs) -> ApiResponse:  # noqa: E501
        """Create role  # noqa: E501

        Creates a role with permissions.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_role_with_http_info(role, async_req=True)
        >>> result = thread.get()

        :param role: (required)
        :type role: Role
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Role, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'role'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_role" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['role'] is not None:
            _body_params = _params['role']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/links+json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['oauth2']  # noqa: E501

        _response_types_map = {
            '201': "Role",
            '401': None,
            '403': None,
            '409': None,
        }

        return self.api_client.call_api(
            '/v1/roles', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def delete_role(self, role_id : Annotated[constr(strict=True, max_length=64, min_length=1), Field(..., description="The identifier of the role.")], **kwargs) -> None:  # noqa: E501
        """Delete role  # noqa: E501

        Remove a role. If a record references the role, that record will not be modified.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_role(role_id, async_req=True)
        >>> result = thread.get()

        :param role_id: The identifier of the role. (required)
        :type role_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the delete_role_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.delete_role_with_http_info(role_id, **kwargs)  # noqa: E501

    @validate_arguments
    def delete_role_with_http_info(self, role_id : Annotated[constr(strict=True, max_length=64, min_length=1), Field(..., description="The identifier of the role.")], **kwargs) -> ApiResponse:  # noqa: E501
        """Delete role  # noqa: E501

        Remove a role. If a record references the role, that record will not be modified.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_role_with_http_info(role_id, async_req=True)
        >>> result = thread.get()

        :param role_id: The identifier of the role. (required)
        :type role_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = [
            'role_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_role" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['role_id'] is not None:
            _path_params['roleId'] = _params['role_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # authentication setting
        _auth_settings = ['oauth2']  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/v1/roles/{roleId}', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def get_role(self, role_id : Annotated[constr(strict=True, max_length=64, min_length=1), Field(..., description="The identifier of the role.")], **kwargs) -> Role:  # noqa: E501
        """Retrieve role  # noqa: E501

        Roles contain a list of permissions that will be applied to any user or resource  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_role(role_id, async_req=True)
        >>> result = thread.get()

        :param role_id: The identifier of the role. (required)
        :type role_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Role
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_role_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.get_role_with_http_info(role_id, **kwargs)  # noqa: E501

    @validate_arguments
    def get_role_with_http_info(self, role_id : Annotated[constr(strict=True, max_length=64, min_length=1), Field(..., description="The identifier of the role.")], **kwargs) -> ApiResponse:  # noqa: E501
        """Retrieve role  # noqa: E501

        Roles contain a list of permissions that will be applied to any user or resource  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_role_with_http_info(role_id, async_req=True)
        >>> result = thread.get()

        :param role_id: The identifier of the role. (required)
        :type role_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Role, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'role_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_role" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['role_id'] is not None:
            _path_params['roleId'] = _params['role_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/links+json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['oauth2']  # noqa: E501

        _response_types_map = {
            '200': "Role",
            '401': None,
            '403': None,
            '404': None,
        }

        return self.api_client.call_api(
            '/v1/roles/{roleId}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def get_roles(self, limit : Annotated[Optional[conint(strict=True, le=100, ge=1)], Field(description="Max number of results to return.")] = None, cursor : Annotated[Optional[StrictStr], Field(description="Continuation cursor for paging.")] = None, filter : Annotated[Optional[constr(strict=True, max_length=128, min_length=0)], Field(description="Filter to search roles by. This is a case insensitive search.")] = None, **kwargs) -> RoleCollection:  # noqa: E501
        """List roles  # noqa: E501

        Get all the account roles. Roles contain a list of permissions that will be applied to any user or resource  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_roles(limit, cursor, filter, async_req=True)
        >>> result = thread.get()

        :param limit: Max number of results to return.
        :type limit: int
        :param cursor: Continuation cursor for paging.
        :type cursor: str
        :param filter: Filter to search roles by. This is a case insensitive search.
        :type filter: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: RoleCollection
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the get_roles_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.get_roles_with_http_info(limit, cursor, filter, **kwargs)  # noqa: E501

    @validate_arguments
    def get_roles_with_http_info(self, limit : Annotated[Optional[conint(strict=True, le=100, ge=1)], Field(description="Max number of results to return.")] = None, cursor : Annotated[Optional[StrictStr], Field(description="Continuation cursor for paging.")] = None, filter : Annotated[Optional[constr(strict=True, max_length=128, min_length=0)], Field(description="Filter to search roles by. This is a case insensitive search.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """List roles  # noqa: E501

        Get all the account roles. Roles contain a list of permissions that will be applied to any user or resource  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_roles_with_http_info(limit, cursor, filter, async_req=True)
        >>> result = thread.get()

        :param limit: Max number of results to return.
        :type limit: int
        :param cursor: Continuation cursor for paging.
        :type cursor: str
        :param filter: Filter to search roles by. This is a case insensitive search.
        :type filter: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(RoleCollection, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'limit',
            'cursor',
            'filter'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_roles" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        if _params.get('cursor') is not None:  # noqa: E501
            _query_params.append(('cursor', _params['cursor']))

        if _params.get('filter') is not None:  # noqa: E501
            _query_params.append(('filter', _params['filter']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/links+json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['oauth2']  # noqa: E501

        _response_types_map = {
            '200': "RoleCollection",
            '401': None,
            '403': None,
        }

        return self.api_client.call_api(
            '/v1/roles', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def update_role(self, role_id : Annotated[constr(strict=True, max_length=64, min_length=1), Field(..., description="The identifier of the role.")], role : Role, **kwargs) -> Role:  # noqa: E501
        """Update role  # noqa: E501

        Updates a role adding or removing permissions.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_role(role_id, role, async_req=True)
        >>> result = thread.get()

        :param role_id: The identifier of the role. (required)
        :type role_id: str
        :param role: (required)
        :type role: Role
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Role
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the update_role_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.update_role_with_http_info(role_id, role, **kwargs)  # noqa: E501

    @validate_arguments
    def update_role_with_http_info(self, role_id : Annotated[constr(strict=True, max_length=64, min_length=1), Field(..., description="The identifier of the role.")], role : Role, **kwargs) -> ApiResponse:  # noqa: E501
        """Update role  # noqa: E501

        Updates a role adding or removing permissions.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_role_with_http_info(role_id, role, async_req=True)
        >>> result = thread.get()

        :param role_id: The identifier of the role. (required)
        :type role_id: str
        :param role: (required)
        :type role: Role
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Role, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'role_id',
            'role'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_role" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['role_id'] is not None:
            _path_params['roleId'] = _params['role_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['role'] is not None:
            _body_params = _params['role']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/links+json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['oauth2']  # noqa: E501

        _response_types_map = {
            '200': "Role",
            '401': None,
            '403': None,
            '404': None,
        }

        return self.api_client.call_api(
            '/v1/roles/{roleId}', 'PUT',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    Authress

    <p> <h2>Introduction</h2> <p>Welcome to the Authress Authorization API. <br>The Authress REST API provides the operations and resources necessary to create records, assign permissions, and verify any user in your platform.</p> <p><ul>   <li>Manage multitenant platforms and create user tenants for SSO connections.</li>   <li>Create records to assign roles and resources to grant access for users.</li>   <li>Check user access control by calling the authorization API at the right time.</li>   <li>Configure service clients to securely access services in your platform.</li> </ul></p> <p>For more in-depth scenarios check out the <a href=\"https://authress.io/knowledge-base\" target=\"_blank\">Authress knowledge base</a>.</p> </p>

    The version of the OpenAPI document: v1
    Contact: support@authress.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from authress.models.extension_collection import ExtensionCollection  # noqa: E501

class TestExtensionCollection(unittest.TestCase):
    """ExtensionCollection unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ExtensionCollection:
        """Test ExtensionCollection
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ExtensionCollection`
        """
        model = ExtensionCollection()  # noqa: E501
        if include_optional:
            return ExtensionCollection(
                extensions = [
                    authress.models.extension.Extension(
                        extension_id = '0', 
                        name = 'Extension: third party example', 
                        created_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        application = authress.models.extension_application.ExtensionApplication(
                            application_id = '', 
                            redirect_urls = [
                                ''
                                ], 
                            links = authress.models.links.Links(
                                self = authress.models.link.Link(
                                    href = '', 
                                    rel = '', ), ), ), 
                        client = authress.models.extension_client.ExtensionClient(
                            client_id = '', 
                            rate_limits = [
                                authress.models.client_rate_limit.ClientRateLimit(
                                    duration = 'PT0S', 
                                    quota = 100, )
                                ], ), 
                        tags = {"environment":"production"}, )
                    ],
                pagination = authress.models.pagination.Pagination(
                    next = authress.models.pagination_next.Pagination_next(
                        cursor = '', ), )
            )
        else:
            return ExtensionCollection(
                extensions = [
                    authress.models.extension.Extension(
                        extension_id = '0', 
                        name = 'Extension: third party example', 
                        created_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        application = authress.models.extension_application.ExtensionApplication(
                            application_id = '', 
                            redirect_urls = [
                                ''
                                ], 
                            links = authress.models.links.Links(
                                self = authress.models.link.Link(
                                    href = '', 
                                    rel = '', ), ), ), 
                        client = authress.models.extension_client.ExtensionClient(
                            client_id = '', 
                            rate_limits = [
                                authress.models.client_rate_limit.ClientRateLimit(
                                    duration = 'PT0S', 
                                    quota = 100, )
                                ], ), 
                        tags = {"environment":"production"}, )
                    ],
        )
        """

    def testExtensionCollection(self):
        """Test ExtensionCollection"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



import base64
import json

class JwtManager:
    def decode(self, token):
        try:
            if token:
                # Extract the payload part of the JWT (the second part)
                payload = token.split('.')[1]
                # Decode the base64url-encoded payload
                decoded_bytes = base64.urlsafe_b64decode(self.pad_base64url(payload))
                # Convert the decoded bytes into a JSON object
                return json.loads(decoded_bytes.decode('utf-8'))
            return None
        except Exception:
            return None

    def pad_base64url(self, data):
        """Pads the base64url string with '=' to make its length a multiple of 4."""
        return data + '=' * (4 - len(data) % 4)


# coding: utf-8

"""
    Authress

    <p> <h2>Introduction</h2> <p>Welcome to the Authress Authorization API. <br>The Authress REST API provides the operations and resources necessary to create records, assign permissions, and verify any user in your platform.</p> <p><ul>   <li>Manage multitenant platforms and create user tenants for SSO connections.</li>   <li>Create records to assign roles and resources to grant access for users.</li>   <li>Check user access control by calling the authorization API at the right time.</li>   <li>Configure service clients to securely access services in your platform.</li> </ul></p> <p>For more in-depth scenarios check out the <a href=\"https://authress.io/knowledge-base\" target=\"_blank\">Authress knowledge base</a>.</p> </p>

    The version of the OpenAPI document: v1
    Contact: support@authress.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Dict, Optional
try:
    from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr, constr, validator
except ImportError:
    from pydantic import BaseModel, Field, StrictBool, StrictStr, constr, validator
from authress.models.connection_conditions import ConnectionConditions
from authress.models.connection_data import ConnectionData
from authress.models.connection_default_connection_properties import ConnectionDefaultConnectionProperties
from authress.models.connection_linking_configuration import ConnectionLinkingConfiguration
from authress.models.connection_user_data_configuration import ConnectionUserDataConfiguration

class Connection(BaseModel):
    """
    Connection
    """
    type: Optional[StrictStr] = 'OAUTH2'
    connection_id: Optional[constr(strict=True, max_length=64, min_length=0)] = Field(default=None, alias="connectionId")
    authentication_url: Optional[constr(strict=True, max_length=256, min_length=1)] = Field(default=None, alias="authenticationUrl", description="Authorization URL of the provider (where the user logs in). For OAuth this is the authorization URL. For SAML, specify the **SSO URL** in this property.")
    token_url: Optional[constr(strict=True, max_length=128, min_length=0)] = Field(default=None, alias="tokenUrl", description="The token exchange url (where we validate the token).")
    issuer_url: Optional[constr(strict=True, max_length=128, min_length=0)] = Field(default=None, alias="issuerUrl", description="The unique identifier tied to the provider's domain used for TLS verification. In OAuth, this is the JWT **iss** property. For SAML, specify the **Entity ID** in this property.")
    provider_certificate: Optional[constr(strict=True, max_length=4096, min_length=0)] = Field(default=None, alias="providerCertificate", description="The Provider's SAML public certificate which can be used to verify the signature in signed SAML assertions from the provider.")
    client_id: Optional[constr(strict=True, max_length=128, min_length=0)] = Field(default=None, alias="clientId", description="Provider's client ID used to verify the token")
    client_secret_id: Optional[constr(strict=True, max_length=32, min_length=0)] = Field(default=None, alias="clientSecretId", description="Provider's client secret identifier used to identify the client secret within your account. Some provider require this property.")
    client_secret: Optional[constr(strict=True, max_length=2048, min_length=0)] = Field(default=None, alias="clientSecret", description="Provider's client secret used to verify the token")
    user_data_configuration: Optional[ConnectionUserDataConfiguration] = Field(default=None, alias="userDataConfiguration")
    data: Optional[ConnectionData] = None
    default_connection_properties: Optional[ConnectionDefaultConnectionProperties] = Field(default=None, alias="defaultConnectionProperties")
    conditions: Optional[ConnectionConditions] = None
    linking_configuration: Optional[ConnectionLinkingConfiguration] = Field(default=None, alias="linkingConfiguration")
    created_time: Optional[datetime] = Field(default=None, alias="createdTime")
    last_updated: Optional[datetime] = Field(default=None, alias="lastUpdated")
    is_active_connection: Optional[StrictBool] = Field(default=None, alias="isActiveConnection")
    tags: Optional[Dict[str, constr(strict=True, max_length=128)]] = Field(default=None, description="The tags associated with this resource, this property is an map. { key1: value1, key2: value2 }")
    __properties = ["type", "connectionId", "authenticationUrl", "tokenUrl", "issuerUrl", "providerCertificate", "clientId", "clientSecretId", "clientSecret", "userDataConfiguration", "data", "defaultConnectionProperties", "conditions", "linkingConfiguration", "createdTime", "lastUpdated", "isActiveConnection", "tags"]

    @validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('OAUTH2', 'SAML2', 'WebAuthN', 'PASSWORD', 'OPENID-LEGACY'):
            raise ValueError("must be one of enum values ('OAUTH2', 'SAML2', 'WebAuthN', 'PASSWORD', 'OPENID-LEGACY')")
        return value

    @validator('connection_id')
    def connection_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^[a-zA-Z0-9-_.:]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-zA-Z0-9-_.:]+$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Connection:
        """Create an instance of Connection from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "created_time",
                            "last_updated",
                            "is_active_connection",
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of user_data_configuration
        if self.user_data_configuration:
            _dict['userDataConfiguration'] = self.user_data_configuration.to_dict()
        # override the default output from pydantic by calling `to_dict()` of data
        if self.data:
            _dict['data'] = self.data.to_dict()
        # override the default output from pydantic by calling `to_dict()` of default_connection_properties
        if self.default_connection_properties:
            _dict['defaultConnectionProperties'] = self.default_connection_properties.to_dict()
        # override the default output from pydantic by calling `to_dict()` of conditions
        if self.conditions:
            _dict['conditions'] = self.conditions.to_dict()
        # override the default output from pydantic by calling `to_dict()` of linking_configuration
        if self.linking_configuration:
            _dict['linkingConfiguration'] = self.linking_configuration.to_dict()
        # set to None if connection_id (nullable) is None
        # and __fields_set__ contains the field
        if self.connection_id is None and "connection_id" in self.__fields_set__:
            _dict['connectionId'] = None

        # set to None if token_url (nullable) is None
        # and __fields_set__ contains the field
        if self.token_url is None and "token_url" in self.__fields_set__:
            _dict['tokenUrl'] = None

        # set to None if issuer_url (nullable) is None
        # and __fields_set__ contains the field
        if self.issuer_url is None and "issuer_url" in self.__fields_set__:
            _dict['issuerUrl'] = None

        # set to None if provider_certificate (nullable) is None
        # and __fields_set__ contains the field
        if self.provider_certificate is None and "provider_certificate" in self.__fields_set__:
            _dict['providerCertificate'] = None

        # set to None if client_id (nullable) is None
        # and __fields_set__ contains the field
        if self.client_id is None and "client_id" in self.__fields_set__:
            _dict['clientId'] = None

        # set to None if client_secret_id (nullable) is None
        # and __fields_set__ contains the field
        if self.client_secret_id is None and "client_secret_id" in self.__fields_set__:
            _dict['clientSecretId'] = None

        # set to None if client_secret (nullable) is None
        # and __fields_set__ contains the field
        if self.client_secret is None and "client_secret" in self.__fields_set__:
            _dict['clientSecret'] = None

        # set to None if user_data_configuration (nullable) is None
        # and __fields_set__ contains the field
        if self.user_data_configuration is None and "user_data_configuration" in self.__fields_set__:
            _dict['userDataConfiguration'] = None

        # set to None if data (nullable) is None
        # and __fields_set__ contains the field
        if self.data is None and "data" in self.__fields_set__:
            _dict['data'] = None

        # set to None if default_connection_properties (nullable) is None
        # and __fields_set__ contains the field
        if self.default_connection_properties is None and "default_connection_properties" in self.__fields_set__:
            _dict['defaultConnectionProperties'] = None

        # set to None if conditions (nullable) is None
        # and __fields_set__ contains the field
        if self.conditions is None and "conditions" in self.__fields_set__:
            _dict['conditions'] = None

        # set to None if linking_configuration (nullable) is None
        # and __fields_set__ contains the field
        if self.linking_configuration is None and "linking_configuration" in self.__fields_set__:
            _dict['linkingConfiguration'] = None

        # set to None if tags (nullable) is None
        # and __fields_set__ contains the field
        if self.tags is None and "tags" in self.__fields_set__:
            _dict['tags'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Connection:
        """Create an instance of Connection from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Connection.parse_obj(obj)

        _obj = Connection.parse_obj({
            "type": obj.get("type") if obj.get("type") is not None else 'OAUTH2',
            "connection_id": obj.get("connectionId"),
            "authentication_url": obj.get("authenticationUrl"),
            "token_url": obj.get("tokenUrl"),
            "issuer_url": obj.get("issuerUrl"),
            "provider_certificate": obj.get("providerCertificate"),
            "client_id": obj.get("clientId"),
            "client_secret_id": obj.get("clientSecretId"),
            "client_secret": obj.get("clientSecret"),
            "user_data_configuration": ConnectionUserDataConfiguration.from_dict(obj.get("userDataConfiguration")) if obj.get("userDataConfiguration") is not None else None,
            "data": ConnectionData.from_dict(obj.get("data")) if obj.get("data") is not None else None,
            "default_connection_properties": ConnectionDefaultConnectionProperties.from_dict(obj.get("defaultConnectionProperties")) if obj.get("defaultConnectionProperties") is not None else None,
            "conditions": ConnectionConditions.from_dict(obj.get("conditions")) if obj.get("conditions") is not None else None,
            "linking_configuration": ConnectionLinkingConfiguration.from_dict(obj.get("linkingConfiguration")) if obj.get("linkingConfiguration") is not None else None,
            "created_time": obj.get("createdTime"),
            "last_updated": obj.get("lastUpdated"),
            "is_active_connection": obj.get("isActiveConnection"),
            "tags": obj.get("tags")
        })
        return _obj





# coding: utf-8

"""
    Authress

    <p> <h2>Introduction</h2> <p>Welcome to the Authress Authorization API. <br>The Authress REST API provides the operations and resources necessary to create records, assign permissions, and verify any user in your platform.</p> <p><ul>   <li>Manage multitenant platforms and create user tenants for SSO connections.</li>   <li>Create records to assign roles and resources to grant access for users.</li>   <li>Check user access control by calling the authorization API at the right time.</li>   <li>Configure service clients to securely access services in your platform.</li> </ul></p> <p>For more in-depth scenarios check out the <a href=\"https://authress.io/knowledge-base\" target=\"_blank\">Authress knowledge base</a>.</p> </p>

    The version of the OpenAPI document: v1
    Contact: support@authress.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
try:
    from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr, constr, validator
except ImportError:
    from pydantic import BaseModel, Field, StrictBool, StrictStr, constr, validator

class ConnectionData(BaseModel):
    """
    ConnectionData
    """
    tenant_id: Optional[constr(strict=True, max_length=128, min_length=0)] = Field(default=None, alias="tenantId")
    developer_account_id: Optional[constr(strict=True, max_length=32, min_length=0)] = Field(default=None, alias="developerAccountId", description="The Developer Account ID associated with the credentials. This is necessary for some providers, such as Apple Login.")
    name: Optional[constr(strict=True, max_length=64, min_length=0)] = Field(default=None, description="The name of this connection when displayed in the Authress management portal")
    supported_content_type: Optional[StrictStr] = Field(default='application/json', alias="supportedContentType", description="URL encode OAuth token parameters - Some authentication APIs don't support JSON, in these cases enable the url encoded form data parameters.")
    oidc_user_endpoint_url: Optional[constr(strict=True, max_length=128, min_length=0)] = Field(default=None, alias="oidcUserEndpointUrl", description="By default, the **sub** claim of the JWT is used to identify the user from this provider. However, not all providers are OpenID compliant. Here you can provide an optional user data endpoint to fetch additional user profile information and an expression to identify a new user ID from available properties.")
    user_id_expression: Optional[constr(strict=True, max_length=128, min_length=0)] = Field(default='{sub}', alias="userIdExpression", description="By default, the **sub** claim of the JWT is used to identify the user from this provider. However, not all providers are OpenID compliant. Here you can provide an optional user expression to identify a new user ID from available properties found from the oidcUserEndpointUrl. (The default is **{sub}**, any claims may be used.)")
    trust_identity_user_id: Optional[StrictBool] = Field(default=False, alias="trustIdentityUserId", description="Authress generates unique user IDs for every user, however if you trust your identity provider to handle unique ID generate across **ALL customers**, then it is safe to reuse the user ID from the provider.")
    __properties = ["tenantId", "developerAccountId", "name", "supportedContentType", "oidcUserEndpointUrl", "userIdExpression", "trustIdentityUserId"]

    @validator('tenant_id')
    def tenant_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^[a-zA-Z0-9-_.:]*$", value):
            raise ValueError(r"must validate the regular expression /^[a-zA-Z0-9-_.:]*$/")
        return value

    @validator('supported_content_type')
    def supported_content_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('application/json', 'application/x-www-form-urlencoded'):
            raise ValueError("must be one of enum values ('application/json', 'application/x-www-form-urlencoded')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ConnectionData:
        """Create an instance of ConnectionData from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if tenant_id (nullable) is None
        # and __fields_set__ contains the field
        if self.tenant_id is None and "tenant_id" in self.__fields_set__:
            _dict['tenantId'] = None

        # set to None if developer_account_id (nullable) is None
        # and __fields_set__ contains the field
        if self.developer_account_id is None and "developer_account_id" in self.__fields_set__:
            _dict['developerAccountId'] = None

        # set to None if name (nullable) is None
        # and __fields_set__ contains the field
        if self.name is None and "name" in self.__fields_set__:
            _dict['name'] = None

        # set to None if supported_content_type (nullable) is None
        # and __fields_set__ contains the field
        if self.supported_content_type is None and "supported_content_type" in self.__fields_set__:
            _dict['supportedContentType'] = None

        # set to None if oidc_user_endpoint_url (nullable) is None
        # and __fields_set__ contains the field
        if self.oidc_user_endpoint_url is None and "oidc_user_endpoint_url" in self.__fields_set__:
            _dict['oidcUserEndpointUrl'] = None

        # set to None if user_id_expression (nullable) is None
        # and __fields_set__ contains the field
        if self.user_id_expression is None and "user_id_expression" in self.__fields_set__:
            _dict['userIdExpression'] = None

        # set to None if trust_identity_user_id (nullable) is None
        # and __fields_set__ contains the field
        if self.trust_identity_user_id is None and "trust_identity_user_id" in self.__fields_set__:
            _dict['trustIdentityUserId'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ConnectionData:
        """Create an instance of ConnectionData from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ConnectionData.parse_obj(obj)

        _obj = ConnectionData.parse_obj({
            "tenant_id": obj.get("tenantId"),
            "developer_account_id": obj.get("developerAccountId"),
            "name": obj.get("name"),
            "supported_content_type": obj.get("supportedContentType") if obj.get("supportedContentType") is not None else 'application/json',
            "oidc_user_endpoint_url": obj.get("oidcUserEndpointUrl"),
            "user_id_expression": obj.get("userIdExpression") if obj.get("userIdExpression") is not None else '{sub}',
            "trust_identity_user_id": obj.get("trustIdentityUserId") if obj.get("trustIdentityUserId") is not None else False
        })
        return _obj





# coding: utf-8

"""
    Authress

    <p> <h2>Introduction</h2> <p>Welcome to the Authress Authorization API. <br>The Authress REST API provides the operations and resources necessary to create records, assign permissions, and verify any user in your platform.</p> <p><ul>   <li>Manage multitenant platforms and create user tenants for SSO connections.</li>   <li>Create records to assign roles and resources to grant access for users.</li>   <li>Check user access control by calling the authorization API at the right time.</li>   <li>Configure service clients to securely access services in your platform.</li> </ul></p> <p>For more in-depth scenarios check out the <a href=\"https://authress.io/knowledge-base\" target=\"_blank\">Authress knowledge base</a>.</p> </p>

    The version of the OpenAPI document: v1
    Contact: support@authress.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
try:
    from pydantic.v1 import BaseModel
except ImportError:
    from pydantic import BaseModel
from authress.models.pagination_next import PaginationNext

class Pagination(BaseModel):
    """
    Details containing how to pagination through the collection. Consists of an optional *next* property that may contain a cursor. Pagination is mutable and the list can change between requests.  # noqa: E501
    """
    next: Optional[PaginationNext] = None
    __properties = ["next"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Pagination:
        """Create an instance of Pagination from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of next
        if self.next:
            _dict['next'] = self.next.to_dict()
        # set to None if next (nullable) is None
        # and __fields_set__ contains the field
        if self.next is None and "next" in self.__fields_set__:
            _dict['next'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Pagination:
        """Create an instance of Pagination from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Pagination.parse_obj(obj)

        _obj = Pagination.parse_obj({
            "next": PaginationNext.from_dict(obj.get("next")) if obj.get("next") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Authress

    <p> <h2>Introduction</h2> <p>Welcome to the Authress Authorization API. <br>The Authress REST API provides the operations and resources necessary to create records, assign permissions, and verify any user in your platform.</p> <p><ul>   <li>Manage multitenant platforms and create user tenants for SSO connections.</li>   <li>Create records to assign roles and resources to grant access for users.</li>   <li>Check user access control by calling the authorization API at the right time.</li>   <li>Configure service clients to securely access services in your platform.</li> </ul></p> <p>For more in-depth scenarios check out the <a href=\"https://authress.io/knowledge-base\" target=\"_blank\">Authress knowledge base</a>.</p> </p>  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: support@authress.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401
import io
import warnings

try:
    from pydantic.v1 import validate_arguments, ValidationError
except ImportError:
    from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

try:
    from pydantic.v1 import Field, StrictStr, conint, constr
except ImportError:
    from pydantic import Field, StrictStr, conint, constr

from typing import Optional


from authress.models.user_identity import UserIdentity
from authress.models.user_identity_collection import UserIdentityCollection

from authress.api_response import ApiResponse
from authress.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class UsersApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        self.api_client = api_client

    @validate_arguments
    def delete_user(self, user_id : Annotated[constr(strict=True, max_length=64, min_length=1), Field(..., description="The user identifier.")], **kwargs) -> None:  # noqa: E501
        """Delete a user  # noqa: E501

        Removes the user, all access the user has been granted through Authress access records, and any related user data. This action is completed asynchronously.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_user(user_id, async_req=True)
        >>> result = thread.get()

        :param user_id: The user identifier. (required)
        :type user_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the delete_user_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.delete_user_with_http_info(user_id, **kwargs)  # noqa: E501

    @validate_arguments
    def delete_user_with_http_info(self, user_id : Annotated[constr(strict=True, max_length=64, min_length=1), Field(..., description="The user identifier.")], **kwargs) -> ApiResponse:  # noqa: E501
        """Delete a user  # noqa: E501

        Removes the user, all access the user has been granted through Authress access records, and any related user data. This action is completed asynchronously.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_user_with_http_info(user_id, async_req=True)
        >>> result = thread.get()

        :param user_id: The user identifier. (required)
        :type user_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = [
            'user_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_user" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['user_id']:
            _path_params['userId'] = _params['user_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # authentication setting
        _auth_settings = ['oauth2']  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/v1/users/{userId}', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def get_user(self, user_id : Annotated[constr(strict=True, max_length=64, min_length=1), Field(..., description="The user identifier.")], **kwargs) -> UserIdentity:  # noqa: E501
        """Retrieve a user  # noqa: E501

        Get the user data associated with a user. The data returned by this endpoint is highly variable based on the source OAuth provider. Avoid depending on undocumented properties.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_user(user_id, async_req=True)
        >>> result = thread.get()

        :param user_id: The user identifier. (required)
        :type user_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: UserIdentity
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_user_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.get_user_with_http_info(user_id, **kwargs)  # noqa: E501

    @validate_arguments
    def get_user_with_http_info(self, user_id : Annotated[constr(strict=True, max_length=64, min_length=1), Field(..., description="The user identifier.")], **kwargs) -> ApiResponse:  # noqa: E501
        """Retrieve a user  # noqa: E501

        Get the user data associated with a user. The data returned by this endpoint is highly variable based on the source OAuth provider. Avoid depending on undocumented properties.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_user_with_http_info(user_id, async_req=True)
        >>> result = thread.get()

        :param user_id: The user identifier. (required)
        :type user_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(UserIdentity, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'user_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_user" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['user_id']:
            _path_params['userId'] = _params['user_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/links+json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['oauth2']  # noqa: E501

        _response_types_map = {
            '200': "UserIdentity",
            '401': None,
            '403': None,
            '404': None,
        }

        return self.api_client.call_api(
            '/v1/users/{userId}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def get_users(self, limit : Annotated[Optional[conint(strict=True, le=100, ge=1)], Field(description="Max number of results to return")] = None, cursor : Annotated[Optional[StrictStr], Field(description="Continuation cursor for paging")] = None, filter : Annotated[Optional[constr(strict=True, max_length=64, min_length=0)], Field(description="Filter to search users by. This is a case insensitive search through every text field.")] = None, tenant_id : Annotated[Optional[str], Field(description="Return only users that are part of the specified tenant. Users can only be part of one tenant, using this parameter will limit returned users that have logged into this tenant.")] = None, **kwargs) -> UserIdentityCollection:  # noqa: E501
        """List users  # noqa: E501

        Returns a paginated user list for the account. The data returned by this endpoint is highly variable based on the source OAuth provider. Avoid depending on undocumented properties.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_users(limit, cursor, filter, tenant_id, async_req=True)
        >>> result = thread.get()

        :param limit: Max number of results to return
        :type limit: int
        :param cursor: Continuation cursor for paging
        :type cursor: str
        :param filter: Filter to search users by. This is a case insensitive search through every text field.
        :type filter: str
        :param tenant_id: Return only users that are part of the specified tenant. Users can only be part of one tenant, using this parameter will limit returned users that have logged into this tenant.
        :type tenant_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: UserIdentityCollection
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_users_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.get_users_with_http_info(limit, cursor, filter, tenant_id, **kwargs)  # noqa: E501

    @validate_arguments
    def get_users_with_http_info(self, limit : Annotated[Optional[conint(strict=True, le=100, ge=1)], Field(description="Max number of results to return")] = None, cursor : Annotated[Optional[StrictStr], Field(description="Continuation cursor for paging")] = None, filter : Annotated[Optional[constr(strict=True, max_length=64, min_length=0)], Field(description="Filter to search users by. This is a case insensitive search through every text field.")] = None, tenant_id : Annotated[Optional[constr(strict=True, max_length=64, min_length=0)], Field(description="Return only users that are part of the specified tenant. Users can only be part of one tenant, using this parameter will limit returned users that have logged into this tenant.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """List users  # noqa: E501

        Returns a paginated user list for the account. The data returned by this endpoint is highly variable based on the source OAuth provider. Avoid depending on undocumented properties.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_users_with_http_info(limit, cursor, filter, tenant_id, async_req=True)
        >>> result = thread.get()

        :param limit: Max number of results to return
        :type limit: int
        :param cursor: Continuation cursor for paging
        :type cursor: str
        :param filter: Filter to search users by. This is a case insensitive search through every text field.
        :type filter: str
        :param tenant_id: Return only users that are part of the specified tenant. Users can only be part of one tenant, using this parameter will limit returned users that have logged into this tenant.
        :type tenant_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(UserIdentityCollection, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'limit',
            'cursor',
            'filter',
            'tenant_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_users" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        if _params.get('cursor') is not None:  # noqa: E501
            _query_params.append(('cursor', _params['cursor']))

        if _params.get('filter') is not None:  # noqa: E501
            _query_params.append(('filter', _params['filter']))

        if _params.get('tenant_id') is not None:  # noqa: E501
            _query_params.append(('tenantId', _params['tenant_id']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/links+json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['oauth2']  # noqa: E501

        _response_types_map = {
            '200': "UserIdentityCollection",
            '401': None,
            '403': None,
        }

        return self.api_client.call_api(
            '/v1/users', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

from __future__ import absolute_import
import json

import jwt
import re
from urllib.parse import urlparse

from authress import rest
from authress.utils import service_client_token_provider, PackageVersionProvider

class TokenVerifier(object):
  def __init__(self, http_client=None):
    self.http_client = http_client
    if http_client is None:
      self.http_client = rest.RESTClientObject()

    self.keyMap = dict()
    self.package_version_provider = PackageVersionProvider()

  def verify_token(self, authressCustomDomain, token, options=None):
    sanitized_domain = re.sub(r"https?://", "", authressCustomDomain)
    completeIssuerUrl = f"https://{sanitized_domain}"
    if token is None:
      raise Exception("Unauthorized")

    try:
      authenticationToken = token
      unverifiedPayload = jwt.decode(authenticationToken, options={"verify_signature": False})
      headers = jwt.get_unverified_header(authenticationToken)
    except jwt.DecodeError:
      try:
        authenticationToken = service_client_token_provider.ServiceClientTokenProvider(token, completeIssuerUrl).get_client_token()
        unverifiedPayload = jwt.decode(authenticationToken, options={"verify_signature": False})
        headers = jwt.get_unverified_header(authenticationToken)
      except Exception as e:
        raise Exception("Unauthorized", f"Invalid Token format: {e}")

    if (unverifiedPayload is None):
      raise Exception("Unauthorized", "Invalid Token or Token not found")

    if (headers is None or 'kid' not in headers):
      raise Exception("Unauthorized", "No KID found in token")
    kid = headers['kid']

    if ('iss' not in unverifiedPayload['iss'] is None):
      raise Exception("Unauthorized", "No Issuer in token")
    issuer = unverifiedPayload['iss']

    if (urlparse(issuer).netloc != urlparse(completeIssuerUrl).netloc):
      raise Exception("Unauthorized", "Issuer does not match")

    # Handle service client checking
    clientIdMatcher = re.match(r"^/v\d/clients/([^/]+)$", urlparse(issuer).path)
    if (clientIdMatcher is not None and clientIdMatcher.group(1) != unverifiedPayload['sub']):
      raise Exception("Unauthorized", "Service ID does not match token sub claim")

    jwk = self.get_public_key(f"{issuer}/.well-known/openid-configuration/jwks?kid={kid}", kid)

    try:
      return jwt.decode(authenticationToken, jwt.api_jwk.PyJWK.from_dict(jwk).key, algorithms=['EdDSA'], options = { 'verify_aud': False })
    except:
      raise Exception("Unauthorized", "Token is invalid")

  def get_public_key(self, jwkKeyListUrl, kid):
    hashKey = f"{jwkKeyListUrl}|{kid}"

    if hashKey in self.keyMap is None:
      self.keyMap[hashKey] = self.get_key_uncached(jwkKeyListUrl, kid)

    try:
      key = self.keyMap[hashKey]
      return key
    except:
      self.keyMap[hashKey] = self.get_key_uncached(jwkKeyListUrl, kid)
      return self.keyMap[hashKey]

  def get_key_uncached(self, jwkKeyListUrl, kid):    
    version = self.package_version_provider.get_version()
    headers = {
      'User-Agent': f'Authress SDK; Python; {version};'
    }
    result = self.http_client.request_with_retries('GET', jwkKeyListUrl, headers=headers)

    for index, key in enumerate(json.loads(result.data)['keys']):
      if key['kid'] == kid:
          return key

    raise Exception("Unauthorized", "The Service Client Access Key is not valid yet or has been deleted. For new Access Keys just created, key validation is cached and can take up to 5 minutes before new keys can be used.")



from __future__ import absolute_import

import unittest
import time

from authress.models import *
from authress.utils import ServiceClientTokenProvider, JwtManager
from authress import AuthressClient

import unittest
from unittest.mock import patch
from urllib.parse import urlparse, parse_qs

class ServiceClientTokenProviderTest(unittest.TestCase):
  def setUp(self):
    pass

  def tearDown(self):
    pass

  def test_get_token(self):
    """Test case for get_token

    Get the token for a client registered as the token provider
    """

    access_key = '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'

    service_client_token_provider = ServiceClientTokenProvider(access_key, "")
    token1 = service_client_token_provider.get_client_token()
    time.sleep(2)
    token2 = service_client_token_provider.get_client_token()
    assert token1 == token2
    pass

  def test_get_token_for_eddsa(self):
    """Test case for get_token for EdDsa

    Get the token for a client registered as the token provider
    """

    access_key = 'CLIENT.KEY.ACCOUNT.MC4CAQAwBQYDK2VwBCIEIIM7npIckfT431rYzEeF+hCqvHogpOllmVSgINwqQv+g'

    service_client_token_provider = ServiceClientTokenProvider(access_key, "")
    token1 = service_client_token_provider.get_client_token()
    time.sleep(2)
    token2 = service_client_token_provider.get_client_token()
    assert token1 == token2
    pass

  def test_get_token_without_access_key(self):
    """Test case for get_token with no access key

    Ignores access keys that are None
    """

    service_client_token_provider = ServiceClientTokenProvider("")
    token1 = service_client_token_provider.get_client_token()
    assert token1 == None
    pass

  def test_should_correctly_parse_parameters_from_authentication_url_object(self):
      access_key = 'clientId.uDeF.a43706ca-9647-40e4-aeae-7dcaa54bbab3.MC4CAQAwBQYDK2VwBCIEIE99LFw2c3DCiYwrY/Qkg1nIDiagoHtdCwb88RxarVYA'
      token_provider = ServiceClientTokenProvider(access_key)

      url = token_provider.generate_user_login_url({'authentication_url': 'https://login.redirect-url.com?client_id=clientId&state=state'}, 'user1')
      parsed_url = urlparse(url)
      query_params = parse_qs(parsed_url.query)
      self.assertEqual(query_params['iss'][0], 'https://login.redirect-url.com/v1/clients/clientId')
      self.assertEqual(query_params['state'][0], 'state')

  def test_should_throw_error_if_state_not_provided(self):
      access_key = 'clientId.uDeF.a43706ca-9647-40e4-aeae-7dcaa54bbab3.MC4CAQAwBQYDK2VwBCIEIE99LFw2c3DCiYwrY/Qkg1nIDiagoHtdCwb88RxarVYA'
      token_provider = ServiceClientTokenProvider(access_key)

      with self.assertRaises(ValueError) as context:
          token_provider.generate_user_login_url({'authentication_url': 'https://login.redirect-url.com?client_id=clientId'}, 'user1')
      self.assertEqual(str(context.exception), 'The state is required to generate a authorization code redirect for is required, and should be present in the authentication_url.')

  def test_should_throw_error_if_client_id_does_not_match(self):
      access_key = 'clientId.uDeF.a43706ca-9647-40e4-aeae-7dcaa54bbab3.MC4CAQAwBQYDK2VwBCIEIE99LFw2c3DCiYwrY/Qkg1nIDiagoHtdCwb88RxarVYA'
      token_provider = ServiceClientTokenProvider(access_key)

      with self.assertRaises(ValueError) as context:
          token_provider.generate_user_login_url({'authentication_url': 'https://login.redirect-url.com?client_id=wrongClientId&state=state'}, 'user1')
      self.assertEqual(str(context.exception), 'The client_id should be specified in the authentication_url. It should match the service client ID.')

  def test_should_throw_error_if_user_id_not_provided(self):
      access_key = 'clientId.uDeF.a43706ca-9647-40e4-aeae-7dcaa54bbab3.MC4CAQAwBQYDK2VwBCIEIE99LFw2c3DCiYwrY/Qkg1nIDiagoHtdCwb88RxarVYA'
      token_provider = ServiceClientTokenProvider(access_key)

      with self.assertRaises(ValueError) as context:
          token_provider.generate_user_login_url({'authentication_url': 'https://login.redirect-url.com?client_id=wrongClientId&state=state'}, None)
      self.assertEqual(str(context.exception), 'The user to generate an authorization code redirect for is required.')

  def test_should_generate_url_with_correct_authorization_code_and_parameters(self):
      access_key = 'clientId.uDeF.a43706ca-9647-40e4-aeae-7dcaa54bbab3.MC4CAQAwBQYDK2VwBCIEIE99LFw2c3DCiYwrY/Qkg1nIDiagoHtdCwb88RxarVYA'
      token_provider = ServiceClientTokenProvider(access_key)

      url = token_provider.generate_user_login_url({'authentication_url': 'https://login.redirect-url.com?client_id=clientId&state=state'}, 'user1')
      parsed_url = urlparse(url)
      query_params = parse_qs(parsed_url.query)

      code = query_params['code'][0]
      self.assertIsNotNone(code)

      jwt_payload = JwtManager().decode(code)
      self.assertEqual(jwt_payload['aud'], 'a43706ca-9647-40e4-aeae-7dcaa54bbab3.accounts.authress.io')
      self.assertEqual(jwt_payload['iss'], 'https://login.redirect-url.com/v1/clients/clientId')
      self.assertEqual(jwt_payload['sub'], 'user1')
      self.assertEqual(jwt_payload['client_id'], 'clientId')
      self.assertIsInstance(jwt_payload['iat'], int)
      self.assertIsInstance(jwt_payload['exp'], int)
      self.assertEqual(jwt_payload['scope'], 'openid')

      self.assertEqual(query_params['iss'][0], 'https://login.redirect-url.com/v1/clients/clientId')
      self.assertEqual(query_params['state'][0], 'state')


# coding: utf-8


from __future__ import absolute_import

import datetime
import json
from urllib.parse import quote, urlencode, urlparse, parse_qs, urlunparse
import jwt
import base64
import json

class ServiceClientTokenProvider(object):
  def __init__(self, access_key, authress_api_url=None):
    self.token = None
    self.decoded_access_key = None
    self.account_id = None
    self.authress_api_url = authress_api_url
    
    if access_key:
      try:
        self.decoded_access_key = json.loads(base64.b64decode(access_key))
        self.account_id = self.decoded_access_key['audience'].split('.')[0]
        self.algorithm = 'RS256'
      except:
        self.account_id = access_key.split('.')[2]
        self.decoded_access_key = {
          'clientId': access_key.split('.')[0],
          'keyId': access_key.split('.')[1],
          'audience': f"{self.account_id}.accounts.authress.io",
          'privateKey': f"-----BEGIN PRIVATE KEY-----\n{access_key.split('.')[3]}\n-----END PRIVATE KEY-----"
        }
        self.algorithm = 'EdDSA'


  def get_client_token(self) -> str:
    if self.decoded_access_key is None:
      return None

    current_token_is_valid = False
    try:
      current_token_is_valid = self.token is not None and jwt.decode(
        self.token, options={"verify_signature": False})['exp'] > (datetime.datetime.utcnow() + datetime.timedelta(seconds=3600)).timestamp()
    except jwt.ExpiredSignatureError:
      current_token_is_valid = False

    if current_token_is_valid is not False:
      return self.token

    issuer_origin = f"https://{self.account_id}.api.authress.io" if (self.authress_api_url == None or self.authress_api_url == '' or '.authress.io' in self.authress_api_url) else self.authress_api_url

    payload = {
      'iss': f"{issuer_origin}/v1/clients/{quote(self.decoded_access_key['clientId'], safe='')}",
      'aud': self.decoded_access_key['audience'],
      'iat': datetime.datetime.utcnow(),
      'exp': datetime.datetime.utcnow() + datetime.timedelta(seconds=86400),
      'sub': self.decoded_access_key['clientId'],
      'client_id': self.decoded_access_key['clientId'],
      'scopes': 'openId'
    }

    jwt_token = jwt.encode(payload, self.decoded_access_key['privateKey'], algorithm=self.algorithm, headers={'kid': self.decoded_access_key['keyId'] })
    self.token = jwt_token if isinstance(jwt_token, str) else jwt_token.decode("utf-8")
    return self.token

  def generate_user_login_url(self, authenticate_response, user_id):
    if not authenticate_response:
        raise ValueError('The authenticate_response is specified in the incoming login request, this should include the configured Authress custom domain.')
    
    parsed_url = urlparse(authenticate_response['authentication_url'])
    parameters = dict(parse_qs(parsed_url.query))
    authress_custom_domain_login_url = authenticate_response['authentication_url']
    client_id = parameters.get('client_id', [None])[0]
    state = parameters.get('state', [None])[0]

    if not state:
        raise ValueError('The state is required to generate a authorization code redirect for is required, and should be present in the authentication_url.')
    if not user_id:
        raise ValueError('The user to generate an authorization code redirect for is required.')
    if not client_id or client_id != self.decoded_access_key['clientId']:
        raise ValueError('The client_id should be specified in the authentication_url. It should match the service client ID.')

    custom_domain_fallback = urlparse(authress_custom_domain_login_url).scheme + "://" + urlparse(authress_custom_domain_login_url).netloc
    issuer = get_issuer(self.authress_api_url or custom_domain_fallback, self.decoded_access_key)

    now = datetime.datetime.utcnow()
    payload = {
        'aud': self.decoded_access_key['audience'],
        'iss': issuer,
        'sub': user_id,
        'client_id': self.decoded_access_key['clientId'],
        'iat': int(now.timestamp()),
        'exp': int((now + datetime.timedelta(seconds=60)).timestamp()),
        'max_age': 60,
        'scope': 'openid'
    }

    if '@' in user_id:
        payload['email'] = user_id
        payload['email_verified'] = True

    code = jwt.encode(payload, self.decoded_access_key['privateKey'], algorithm=self.algorithm, headers={'kid': self.decoded_access_key['keyId'], 'typ': 'oauth-authz-req+jwt'})

    parsed_url = urlparse(authress_custom_domain_login_url)
    query_params = dict(parse_qs(parsed_url.query))
    query_params['code'] = code
    query_params['iss'] = issuer
    query_params['state'] = state

    new_query_string = urlencode(query_params, doseq=True)
    new_url = urlunparse(parsed_url._replace(query=new_query_string))
    return new_url

def get_issuer(domain, decoded_access_key):
    return f"{domain}/v1/clients/{decoded_access_key['clientId']}"


# coding: utf-8

"""
    Authress

    <p> <h2>Introduction</h2> <p>Welcome to the Authress Authorization API. <br>The Authress REST API provides the operations and resources necessary to create records, assign permissions, and verify any user in your platform.</p> <p><ul>   <li>Manage multitenant platforms and create user tenants for SSO connections.</li>   <li>Create records to assign roles and resources to grant access for users.</li>   <li>Check user access control by calling the authorization API at the right time.</li>   <li>Configure service clients to securely access services in your platform.</li> </ul></p> <p>For more in-depth scenarios check out the <a href=\"https://authress.io/knowledge-base\" target=\"_blank\">Authress knowledge base</a>.</p> </p>

    The version of the OpenAPI document: v1
    Contact: support@authress.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from authress.api.extensions_api import ExtensionsApi  # noqa: E501


class TestExtensionsApi(unittest.TestCase):
    """ExtensionsApi unit test stubs"""

    def setUp(self) -> None:
        self.api = ExtensionsApi()  # noqa: E501

    def tearDown(self) -> None:
        pass

    def test_create_extension(self) -> None:
        """Test case for create_extension

        Create extension  # noqa: E501
        """
        pass

    def test_delete_extension(self) -> None:
        """Test case for delete_extension

        Delete extension  # noqa: E501
        """
        pass

    def test_get_extension(self) -> None:
        """Test case for get_extension

        Retrieve extension  # noqa: E501
        """
        pass

    def test_get_extensions(self) -> None:
        """Test case for get_extensions

        List extensions  # noqa: E501
        """
        pass

    def test_login(self) -> None:
        """Test case for login

        OAuth Authorize  # noqa: E501
        """
        pass

    def test_request_token(self) -> None:
        """Test case for request_token

        OAuth Token  # noqa: E501
        """
        pass

    def test_update_extension(self) -> None:
        """Test case for update_extension

        Update extension  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8
from __future__ import absolute_import

import json
import re

from authress.api.applications_api import ApplicationsApi
from authress.api.connections_api import ConnectionsApi
from authress.api.extensions_api import ExtensionsApi
from authress.api.groups_api import GroupsApi
from authress.api.invites_api import InvitesApi
from authress.api.roles_api import RolesApi
from authress.api.tenants_api import TenantsApi
from authress.api.users_api import UsersApi

# python 2 and python 3 compatibility library
from authress.http_client import HttpClient
from authress.api.access_records_api import AccessRecordsApi
from authress.api.accounts_api import AccountsApi
from authress.api.resource_permissions_api import ResourcePermissionsApi
from authress.api.service_clients_api import ServiceClientsApi
from authress.api.user_permissions_api import UserPermissionsApi
from authress.api import token_verifier

class AuthressClient(object):
    def __init__(self, authress_api_url=None, service_client_access_key=None, user_agent=None):
        self._host = authress_api_url if authress_api_url.startswith('http') else f"https://{authress_api_url}"
        self._host = re.sub(r'/+$', '', self._host)
        
        self._http_client = HttpClient(host=self._host, access_key=service_client_access_key, user_agent=user_agent)
        self._token_verifier = token_verifier.TokenVerifier(http_client=self._http_client)

    def set_token(self, token: str):
        self._http_client.set_token(token)

    def get_client_token(self) -> str:
        """Generates a Service Client Machine JWT to be used for securing machine to machine requests."""
        return self._http_client._get_client_token()

    def verify_token(self, token: str) -> object:
      """Verify a user access token

        On successful verification the response is the decoded user identity JWT.
        On failure this raises an exception

        Note: This method makes HTTP calls to fetch public keys for token signature verification

        :param string token: (required) The user's access token to verify
        :raises: :class:`Exception`: Unauthorized

        :return: UserIdentityJWT
        """
      return self._token_verifier.verify_token(self._host, token)

    @property
    def user_permissions(self) -> UserPermissionsApi:
        """Gets the User permissions API to make HTTP Requests with."""
        return UserPermissionsApi(api_client=self._http_client)

    @property
    def users(self) -> UsersApi:
        """Gets the Users API to make HTTP Requests with."""
        return UsersApi(api_client=self._http_client)

    @property
    def access_records(self) -> AccessRecordsApi:
        """Gets the Access Records API to make HTTP Requests with."""
        return AccessRecordsApi(api_client=self._http_client)

    @property
    def accounts(self) -> AccountsApi:
        """Gets the Authress Admin API to make HTTP Requests with."""
        return AccountsApi(api_client=self._http_client)

    @property
    def applications(self) -> ApplicationsApi:
        """Gets the Applications API to make HTTP Requests with."""
        return ApplicationsApi(api_client=self._http_client)

    @property
    def connections(self) -> ConnectionsApi:
        """Gets the Connections API to make HTTP Requests with."""
        return ConnectionsApi(api_client=self._http_client)

    @property
    def extensions(self) -> ExtensionsApi:
        """Gets the Extensions API to make HTTP Requests with."""
        return ExtensionsApi(api_client=self._http_client)

    @property
    def groups(self) -> GroupsApi:
        """Gets the Groups API to make HTTP Requests with."""
        return GroupsApi(api_client=self._http_client)

    @property
    def invites(self) -> InvitesApi:
        """Gets the Invites API to make HTTP Requests with."""
        return InvitesApi(api_client=self._http_client)

    @property
    def resource_permissions(self) -> ResourcePermissionsApi:
        """Gets the ResourcePermissions API to make HTTP Requests with."""
        return ResourcePermissionsApi(api_client=self._http_client)

    @property
    def roles(self) -> RolesApi:
        """Gets the Roles API to make HTTP Requests with."""
        return RolesApi(api_client=self._http_client)

    @property
    def tenants(self) -> TenantsApi:
        """Gets the Tenants API to make HTTP Requests with."""
        return TenantsApi(api_client=self._http_client)

    @property
    def service_clients(self) -> ServiceClientsApi:
        """Gets the Service Clients API to make HTTP Requests with."""
        return ServiceClientsApi(api_client=self._http_client)


# coding: utf-8

"""
    Authress

    <p> <h2>Introduction</h2> <p>Welcome to the Authress Authorization API. <br>The Authress REST API provides the operations and resources necessary to create records, assign permissions, and verify any user in your platform.</p> <p><ul>   <li>Manage multitenant platforms and create user tenants for SSO connections.</li>   <li>Create records to assign roles and resources to grant access for users.</li>   <li>Check user access control by calling the authorization API at the right time.</li>   <li>Configure service clients to securely access services in your platform.</li> </ul></p> <p>For more in-depth scenarios check out the <a href=\"https://authress.io/knowledge-base\" target=\"_blank\">Authress knowledge base</a>.</p> </p>

    The version of the OpenAPI document: v1
    Contact: support@authress.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, constr, validator

try:
    from pydantic.v1 import BaseModel, Field, constr, validator
except ImportError:
    from pydantic import BaseModel, Field, constr, validator

class AuthenticationTokenConfiguration(BaseModel):
    """
    AuthenticationTokenConfiguration
    """
    access_token_duration: Optional[constr(strict=True)] = Field(default='PT24H', alias="accessTokenDuration", description="How long should Authress generated access tokens (JWTs) last for in minutes. This controls how often tokens expiry (*exp*). The default is 24 hours. The minimum is one minute and the max is twenty-four hours.")
    session_duration: Optional[constr(strict=True)] = Field(default='P30D', alias="sessionDuration", description="How long should user authentication sessions last for in minutes. This controls how often users are forced to log in again. User sessions are optimized to provide the best user experience for your application. The default is 90 days. The minimum is one minute and the max is 90 days.")
    __properties = ["accessTokenDuration", "sessionDuration"]

    @validator('access_token_duration')
    def access_token_duration_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^P(?!$)(\d+Y)?(\d+M)?(\d+W)?(\d+D)?(T(?=\d+[HMS])(\d+H)?(\d+M)?(\d+S)?)?$", value):
            raise ValueError(r"must validate the regular expression /^P(?!$)(\d+Y)?(\d+M)?(\d+W)?(\d+D)?(T(?=\d+[HMS])(\d+H)?(\d+M)?(\d+S)?)?$/")
        return value

    @validator('session_duration')
    def session_duration_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^P(?!$)(\d+Y)?(\d+M)?(\d+W)?(\d+D)?(T(?=\d+[HMS])(\d+H)?(\d+M)?(\d+S)?)?$", value):
            raise ValueError(r"must validate the regular expression /^P(?!$)(\d+Y)?(\d+M)?(\d+W)?(\d+D)?(T(?=\d+[HMS])(\d+H)?(\d+M)?(\d+S)?)?$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AuthenticationTokenConfiguration:
        """Create an instance of AuthenticationTokenConfiguration from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if access_token_duration (nullable) is None
        # and __fields_set__ contains the field
        if self.access_token_duration is None and "access_token_duration" in self.__fields_set__:
            _dict['accessTokenDuration'] = None

        # set to None if session_duration (nullable) is None
        # and __fields_set__ contains the field
        if self.session_duration is None and "session_duration" in self.__fields_set__:
            _dict['sessionDuration'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AuthenticationTokenConfiguration:
        """Create an instance of AuthenticationTokenConfiguration from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return AuthenticationTokenConfiguration.parse_obj(obj)

        _obj = AuthenticationTokenConfiguration.parse_obj({
            "access_token_duration": obj.get("accessTokenDuration") if obj.get("accessTokenDuration") is not None else 'PT24H',
            "session_duration": obj.get("sessionDuration") if obj.get("sessionDuration") is not None else 'P30D'
        })
        return _obj


